'''
    RemoveDups is a program to remove duplicate polygons to avoid bad import data
    Copyright (C) 2010 Tim Sheerman-Chase

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 or version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import libxml2

def ReadXmlTag(keypair):
	attrMap = {}
	attr = keypair.get_properties()

	#Read attributes
	while attr is not None:
		#print attr.name, attr.content
		attrMap[attr.name] = attr.content
		attr = attr.next
	return attrMap['k'],attrMap['v']

def ReadXmlNodeTag(node):
	attr = node.get_properties()
	#Read attributes
	while attr is not None:
		if attr.name == 'ref':
			return int(attr.content)
		attr = attr.next
	return None

def ReadXmlRelationMember(member):
	attrMap = {}
	attr = member.get_properties()

	#Read attributes
	while attr is not None:
		#print attr.name, attr.content
		attrMap[attr.name] = attr.content
		attr = attr.next
	return attrMap['type'],attrMap['ref'],attrMap['role']

def ReadXmlNode(node):
	out = {}
	out['tags'] = {}
	attr = node.get_properties()

	#Read attributes
	while attr is not None:
		#print attr.name, attr.content
		if attr.name == 'lat' or attr.name == 'lon':
			out[attr.name] = float(attr.content)
		else:
			out[attr.name] = str(attr.content)
		attr = attr.next

	#Read tags
	keys = node.children
	while keys is not None:
		if keys.type == "element" and keys.name == "tag":
			#print keys.name, keys.content
			key, value = ReadXmlTag(keys)
			out['tags'][key] = str(value)
		keys = keys.next
	return out

def ReadXmlWay(way):
	out = {}
	out['tags'] = {}
	out['nd'] = []
	attr = way.get_properties()

	#Read attributes
	while attr is not None:
		#print attr.name, attr.content
		out[attr.name] = str(attr.content)
		attr = attr.next

	#Read tags
	keys = way.children
	while keys is not None:
		if keys.type == "element" and keys.name == "tag":
			#print keys.name, keys.content
			key, value = ReadXmlTag(keys)
			out['tags'][key] = str(value)

		if keys.type == "element" and keys.name == "nd":
			#print keys.name, keys.content
			out['nd'].append(ReadXmlNodeTag(keys))

		keys = keys.next
	return out

def ReadXmlRelation(relation):
	out = {}
	out['tags'] = {}
	out['nodeMembers'] = []
	out['wayMembers'] = []
	out['relationMembers'] = []

	attr = relation.get_properties()
	#Read attributes
	while attr is not None:
		#print attr.name, attr.content
		out[attr.name] = str(attr.content)
		attr = attr.next

	#Read tags
	keys = relation.children
	while keys is not None:
		if keys.type == "element" and keys.name == "tag":
			#print keys.name, keys.content
			key, value = ReadXmlTag(keys)
			out['tags'][key] = str(value)

		if keys.type == "element" and keys.name == "member":
			#print keys.name, keys.content
			memberType, ref, role = ReadXmlRelationMember(keys)
			if memberType == "node":	
				out['nodeMembers'].append([role, ref])
			if memberType == "way":	
				out['wayMembers'].append([role, ref])
			if memberType == "relation":	
				out['relationMembers'].append([role, ref])

		keys = keys.next
	return out

def SubstituteNodesIntoWays(nodes,ways):
	for wayId, way in ways.iteritems():
		newList = []
		if 'nd' not in way:
			print way
		for ref in way['nd']:
			newList.append(nodes[ref])
		way['nd'] = newList


def ParseOsm06(text):
	doc = libxml2.parseDoc(text)
	root = doc.children
	#print root
	#iterate over children of verse

	nodes = {}
	ways = {}
	relations = {}
	child = root.children
	while child is not None:
		#print child
		if child.type == "element":
			if child.name == "node":
				nativeNode = ReadXmlNode(child)
				nodes[int(nativeNode["id"])] = nativeNode
			if child.name == "way":
				nativeWay = ReadXmlWay(child)
				ways[int(nativeWay["id"])] = nativeWay
			if child.name == "relation":
				nativeRelation = ReadXmlRelation(child)
				relations[int(nativeRelation["id"])] = nativeRelation
	
			#print "\tAn element with ", child.lsCountNode(), "child(ren)"
			#print "\tAnd content", repr(child.content)
		child = child.next
	doc.freeDoc()
	return nodes, ways, relations

