'''
    RemoveDups is a program to remove duplicate polygons to avoid bad import data
    Copyright (C) 2010 Tim Sheerman-Chase

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 or version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

from ReadOsm06 import *
from WriteOsm06 import *
from shapely.geometry import Polygon
import getopt
import sys

def FilterWays(ways, filterTag):
	out = {}
	for wayId in ways.keys():
		#print wayId, ways[wayId]
		tags = ways[wayId]['tags']
		if filterTag in tags:
			buildingVal = tags[filterTag]
			if buildingVal == "no" or buildingVal == "0" or buildingVal == "false":
				continue
			out[wayId] = ways[wayId]
	return out

def AddShapelyData(nodes,ways):
	for wayId in ways:
		way = ways[wayId]
		temp = []
		for n in way['nd']:
			temp.append((nodes[n]['lon'],nodes[n]['lat']))
		if len(temp) <= 2: continue
		shapelyPoly = Polygon(temp)
		way['shapely'] = shapelyPoly

def usage():
	print "--help (-h)     Print help message"
	print "--in (-i)       Specify OSM file to filter"
	print "--baseline (-b) Specify OSM file containing existing polygons"
	print "--out (-o)      Output filename to save OSM polygons"

if __name__ == '__main__':

	try:
		opts, args = getopt.getopt(sys.argv[1:], "hi:o:b:", ["help", "in=", "out=","baseline="])
	except getopt.GetoptError, err:
		# print help information and exit:
		print str(err) # will print something like "option -a not recognized"
		usage()
		sys.exit(2)

	toFilter = None#"su85se.osm"
	existing = None#"existing_su85se.osm"
	outputFilename = "no_dups.osm"
	#toFilter = "local.osm"
	#existing = "localexist.osm"

	for o, a in opts:
		if o in ("-i","--in"):
			toFilter = a
		elif o in ("-h", "--help"):
			usage()
			sys.exit()
		elif o in ("-o", "--out"):
			outputFilename = a
		elif o in ("-b", "--baseline"):
			existing = a
		else:
			assert False, "unhandled option "+o

	if toFilter == None:
		print "Error: input file must be specified"
		exit(-1)
	if existing == None:
		print "Error: baseline polygon file must be specified"
		exit(-2)

	print "Reading "+toFilter
	nodes1, ways1, relations1 = ParseOsm06(open(toFilter,"rt").read())

	print "Reading "+existing
	nodes2, ways2, relations2 = ParseOsm06(open(existing,"rt").read())

	buildingWays1 = FilterWays(ways1, 'building')
	buildingWays2 = FilterWays(ways2, 'building')

	AddShapelyData(nodes1, buildingWays1)
	AddShapelyData(nodes2, buildingWays2)
	overlapping1,overlapping2 = [],[]

	print "Check for overlaps"
	for wayId1 in buildingWays1:
		for wayId2 in buildingWays2:
			way1 = buildingWays1[wayId1]			
			way2 = buildingWays2[wayId2]

			if 'shapely' not in way1 or 'shapely' not in way2:
				continue

			if way1['shapely'].intersects(way2['shapely']):
				#print "Overlap between",wayId1, wayId2
				overlapping1.append(wayId1)
				overlapping2.append(wayId2)

	print "Remove "+str(len(overlapping1))+" duplicates"
	for wayId in overlapping1:
		#Delete corresponding nodes
		if wayId in ways1:
			way = ways1[wayId]
			for nodeId in way['nd']:
				if nodeId in nodes1:
					del nodes1[nodeId]

		#Delete ways
		if wayId in buildingWays1:
			del buildingWays1[wayId]
		if wayId in ways1:
			del ways1[wayId]

	print "Remove shapely data"
	for wayId in ways1.keys():
		way = ways1[wayId]
		if 'shapely' in way:
			del way['shapely']

	print "Save Output to "+outputFilename
	WriteOsm06(nodes1, ways1, relations1, outputFilename)

