'''
    Mapseg, a program for segmenting maps and producing OSM polygons
    Copyright (C) 2010 Tim Sheerman-Chase

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 or version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import Image
import os
import sys
import time
import os.path
from selectcolour import *
from TransformToWorld import *
from globalmaptiles import GlobalMercator
from GroupOverlappingPolys import *
from exportosm import *
from OSGB import *
from OrthoPolygons import *
from SimplifyPolys import *

if __name__ == '__main__':

	print "Mapseg by Tim Sheerman-Chase"
	print "See COPYING file for copyright info. See README for usage."
	print "This program comes with ABSOLUTELY NO WARRANTY."

	fullInputImage = None
	outFilename = None
	if len(sys.argv)>1:
		fullInputImage = sys.argv[1]
	if fullInputImage == None:
		if 0:
			fullInputImage = "test.png"
			bounds = GlobalMercator().TileLatLonBounds(65322,43739,17) #test.png
			bounds = [-bounds[0],bounds[1],-bounds[2],bounds[3]] #Not sure why I have to hack the sign on latitude...
			coords = "wgs84"
			smoothThreshold = 2.0

		if 0:
			fullInputImage = "cropped.png"
			bounds=( 151316.0, 495558.0, 150669.0, 496316.0 )
			coords = "gbos"
			smoothThreshold = 1.3

	if 1:
		if fullInputImage == None:
			fullInputImage = "su95se.png"
	
		(filePath,inputImage) = os.path.split(fullInputImage)
		(baseName, fileExt) = os.path.splitext(inputImage)
		outFilename = baseName + ".osm"
		tileCode = inputImage[0:7]
		gridStr = str.upper(tileCode[0:2])
		subTileCodeN = str.lower(tileCode[4:5])
		subTileCodeE = str.lower(tileCode[5:6])
		(e,n) = parse_grid(gridStr)
		e = e + int(tileCode[2:3]) * 10000
		n = n + int(tileCode[3:4]) * 10000
		if subTileCodeN == 'n': n = n + 5000
		if subTileCodeE == 'e': e = e + 5000
		bounds=( n+5000, e, n, e+5000 ) #minLat, minLon, maxLat, maxLon
		coords = "gbos"
		smoothThreshold = 1.3

	print "bounds",bounds

	#Original images are colour indexed. Convert to RGB if required.
	im = Image.open(fullInputImage)
	print "input image mode",im.mode
	if im.mode != 'RGB':
		im = im.convert('RGB')
	if outFilename == None:
		outFilename = "export.osm"
	print "output filename",outFilename

	print "Selecting by Colour"
	#Select colour areas
	colSelect = DistSelectColour(im,(246,213,139),50.0)
	pixSelect = ThresholdImageBelowT(colSelect,20.0)
	pixSelect = Normalise(pixSelect)


	#Find polygon shapes
	if 1:
		print "Creating Edges"
		startEdgeTime = time.time()
		edges = SelectionToEdge(pixSelect)
		endEdgeTime = time.time()
		print "Found",len(edges),"in",endEdgeTime - startEdgeTime,"sec"

		print "Creating Polygons"
		startPolyTime = time.time()
		paths = EdgeFragmentsToPolygons(edges, im.size)
		endPolyTime = time.time()
		print "Found",len(paths),"in",endPolyTime - startPolyTime,"sec"


	print "Simplifying Polygons"
	simplifiedPolysSmooth = SimplifyPolysDpMultiStart(paths,smoothThreshold)
	print len(simplifiedPolysSmooth)

	if 1:
		print "Ortho polygons"
		simplifiedPolysSmooth = OrthoPolygons(simplifiedPolysSmooth)
		#print simplifiedPolysSmooth
	
	print "Checking for fragments and adding tags"
	for poly in simplifiedPolysSmooth:
		if len(poly['path']) <= 4: #Detect polygons with three or less points (4 including common start/end node)
			poly['meta']['mapseg:fragment'] = 'FIXME'
		poly['meta']['building'] = 'yes'
		poly['meta']['source'] = 'Auto_OS_OpenData_StreetView'

	print "Merging Polygons"
	shapely = ConvertPolyToShapely(simplifiedPolysSmooth)
	intersectingPolys = GroupIntersecting(shapely)
	outerInnerPoly = FindOuterPolys(intersectingPolys)
	simplyPolys = ConvertShapelyToPoly(outerInnerPoly)

	print "Transforming Polygons"
	worldPolys = TransformPolysToWorld(simplyPolys, im.size, bounds)
	if coords == "gbos":
		worldPolys = TransformOsgbToWgs84(worldPolys)

	print "Exporting"
	ExportOsm(worldPolys, outFilename)

	#print worldPolys



